/*
  Copyright (C) 2013 Grégory Soutadé
  
  This file is part of gPass.
  
  gPass is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  gPass is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with gPass.  If not, see <http://www.gnu.org/licenses/>.
*/

var {Cc, Ci} = require("chrome");

var hmac = Cc["@mozilla.org/security/hmac;1"]
    .createInstance(Ci.nsICryptoHMAC);

function hmac_init(key) {
    var keyObject = Cc["@mozilla.org/security/keyobjectfactory;1"]
                .getService(Ci.nsIKeyObjectFactory)
                .keyFromString(Ci.nsIKeyObject.HMAC, key);
    hmac.init(hmac.SHA256, keyObject);
}

function hmac_digest(message) {
    var data = new Uint8Array(message.length);
    for(i=0; i<message.length; i++)
	data[i] = message.charCodeAt(i);
    hmac.update(data, data.length);
    res = hmac.finish(false);
    hmac.reset();
    return res;
}

exports.pkdbf2 = {
    
    pkdbf2 : function(password, salt, iterations, outlen) {
	var result = "";
	var temp = "";
	var temp2 = "";
	var temp_res = "";
	var temp_res2 = "";
	    
	hmac_init(password);
	for (i=1; result.length < outlen; i++)
	{
	    temp = hmac_digest(salt + 
	    		       String.fromCharCode((i & 0xff000000) >> 24) +
	    		       String.fromCharCode((i & 0x00ff0000) >> 16) +
	    		       String.fromCharCode((i & 0x0000ff00) >>  8) +
	    		       String.fromCharCode((i & 0x000000ff) >>  0)
			      );
	    temp_res = temp;
	    
	    for(a=1; a<iterations; a++)
	    {
		temp2 = hmac_digest(temp);
		temp_res2 = "";
		for(b = 0; b<temp_res.length; b++)
		    temp_res2 += String.fromCharCode(temp_res.charCodeAt(b) ^ temp2.charCodeAt(b));
		temp_res = temp_res2;
		temp = temp2;
	    }

	    result += temp_res;
	}

	return result.substr(0, outlen);
    }
};